// VinciaCommon.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2010 W.T. Giele, D.A. Kosower, P.Z. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains header information for the auxiliary helper 
// functions for VINCIA, including the AlphaQCD class, which inherits
// from Pythia8::AlphaStrong, but adds a bit of extra functionality.

#ifndef VINCIACOMMON_H
#define VINCIACOMMON_H

// Include Pythia 8 headers and name space

#include "Basics.h"
#include "Event.h"
#include "PythiaStdlib.h"

using namespace Pythia8;

// ------------ GLOBAL VINCIA CONSTANTS AND FUNCTIONS --------------------
namespace Vincia {
  
  //**************************************************************************
  
  // VINCIA Version number - do not modify
  
  const double version = 1.024;

  // *********************************************************************
  
  // Globally accessible functions and constants in the Vincia namespace
  
  // Global numerical precision target
  const double TINY    = 1.0e-5;

  // Color factors in Vincia normalization
  const double CA=3.0;
  const double CF=8.0/3.0;
  const double TR=1.0;
  const double NC=3.0;

  // Indices of antenna functions
  const int iqqEmit  = 0;
  const int iqgEmit  = 1;
  const int iqgSplit = 2;
  const int iggEmit  = 3;
  const int iggSplit = 4;

  // Mathematical constants
  const double gammaE=0.577215664901532860606512090082402431042;
  
  // Max index for antenna coefficients
  const int LaurentIndexMax  = 10; 
  
  // Utilities for printing out VINCIA Messages
  void printErr(string,string);
  void printOut(string,string);
  
  // Utilities for converting data to strings
  string num2str(int,int width=7) ;
  string num2str(double,int width=9) ;
  string bool2str(bool) ;
  
  // A few useful auxiliary functions
  double m(const Vec4&);
  double m2(const Vec4&);
  double m2(const Vec4&, const Vec4&, const Vec4&);
  double m2(const Vec4&, const Vec4&, const Vec4&, const Vec4&);
  double m2(const Particle&, const Particle&, const Particle&);
  void Vec4toD4(const Vec4&, double*);
  void Vec4toMG(const Vec4&, double*);
  
  // Utilities to find and sort a bunch of partons
  int colorSort(Event&, vector<int>&);
  int findDaughters(Event&, int, vector<int>&);
  // Function to return whether parton i in event has iAncestor in its history
  bool isAncestor(Event& event, int i, int iAncestor);
  
  // Math support tools
  double zbrent(double (*)(double), double, double, double, double);
  double integrateGauss( double (*)(double), double, double, double);
  void makeSpline(vector<double>&, vector<double>&, vector<double>&);
  double getSpline(const double, vector<double>&, vector<double>&, 
		   vector<double>&);
  
  // Math support functions
  double Li2(const double,const double kmax=100.0,const double xerr=TINY);
  double lnGamma(const double);
  double Gamma(const double);
  int factorial(const int);
  int binomial(const int,int);
  
  //**************************************************************************
  
  // A simple class to contain a spline
  
  class spline {
  public: 
    
    // constructor
    spline() {};
    
    // Members
    vector<double> x, y, d2;
  };
  
  //**************************************************************************

  // The AlphaQCD class inherits from Pythia8::AlphaStrong, but can be 
  // expanded to include additional functionality without affecting 
  // Pythia 8's internally used couplings. 
  
  class AlphaQCD : public Pythia8::AlphaStrong {
    
  public: 
    
    // Constructors.
    AlphaQCD() { isInit=false; }
    AlphaQCD(double valueIn, int orderIn = 1, int verbose = 1) { 
      if (valueIn <= 0.0) printOut("AlphaQCD","Error. Attempting to initialize with aS(mZ) = "+num2str(valueIn)+" order = "+num2str(orderIn));
      else if (verbose >= 9) {
	printOut("AlphaQCD","initializing with aS(mZ) = "
				 +num2str(valueIn)+" order = "+num2str(orderIn));
	init( valueIn, orderIn) ; 
      }
    }
    
    // ---- ORDINARY FUNCTIONALITY INHERITED FROM Pythia8::AlphaStrong ----
    // ... 
    
    // ---- Additional Functionality ----
    
    // alpha explicitly of Q and Q2, respectively
    double alphaSofQ(double Q) {
      return alphaS(pow2(Q));
    }
    double alphaSofQ2(double Q2) {
      return alphaS(Q2);
    }
    
  protected: 
    
  private:

  }; // End class AlphaQCD
  
} // End VINCIA namespace
#endif
